package com.muyu.marketing.domain.model;


import com.fasterxml.jackson.annotation.JsonFormat;
import com.muyu.marketing.domain.req.ActivityTeamProductSkuInfoUpdReq;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;

/**
 * 团购活动修改模型
 *
 * @author DongZeLiang
 * @date 2024-11-26 09:38
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class ActivityTeamInfoUpdModel {

    /**
     * 活动ID
     */
    private Long teamId;
    /**
     * 商品ID
     */
    private Long productId;
    /**
     * 拼团名称
     */
    private String name;

    /**
     * 商品活动图
     */
    private String productImage;
    /**
     * 活动简介
     */
    private String introduction;
    /**
     * 详情
     */
    private String content;
    /**
     * 单位
     */
    private String unit;

    /**
     * 轮播图
     */
    private String imageList;
    /**
     * 活动结束时间
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private Date endTime;
    /**
     * 策略类型
     */
    private String strategyType;

    /**
     * 商品规格
     */
    private ActivityTeamProductUpdCheckModel skuList;

    /**
     * 活动状态
     */
    private String status;

    /**
     * 策略ID
     */
    private Long strategyId;

    /** 拼团活动req 构造 model*/
    public static ActivityTeamInfoUpdModel reqConvertModel(ActivityTeamProductSkuInfoUpdReq skuInfoUpdReq){
        return ActivityTeamInfoUpdModel.builder()
               .teamId(skuInfoUpdReq.getTeamId())
               .productId(skuInfoUpdReq.getProductId())
               .name(skuInfoUpdReq.getName())
               .introduction(skuInfoUpdReq.getIntroduction())
               .content(skuInfoUpdReq.getContent())
               .unit(skuInfoUpdReq.getUnit())
               .imageList(skuInfoUpdReq.getImageList())
               .endTime(skuInfoUpdReq.getEndTime())
               .strategyType(skuInfoUpdReq.getStrategyType())
               .skuList(ActivityTeamProductUpdCheckModel
                       .reqConvertModel(skuInfoUpdReq.getSukList()))   //req skuList 构造 model
               .status(skuInfoUpdReq.getStatus())
               .strategyId(skuInfoUpdReq.getStrategyId())
               .build();
    }
}
