package com.muyu.marketing.domain.resp;

/**
 * 添加拼团活动
 */

import com.fasterxml.jackson.annotation.JsonFormat;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.model.ActivityTeamInfoSelectModel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 查询拼团活动信息
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class ActivityTeamInfoResp extends BaseEntity {


    /**
     * 拼团活动id
     */
    private Long teamId;
    /**
     * 拼团名称
     */
    private String name;
    /**
     * 商品ID
     */
    private Long productId;
    /**
     * 商品价格
     */
    private BigDecimal productPrice;

    /**
     * 商品活动图
     */
    private String productImage;
    /**
     * 活动简介
     */
    private String introduction;
    /**
     * 单位
     */

    private String unit;
    /**
     * 轮播图
     */

    private String imageList;
    /**
     * 活动结束时间
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd")
    private Date endTime;
    /**
     * 活动排序
     */
    private Long sort;
    /**
     * 商品SkU集合
     */
    private List<ProjectFindSkuInfoResp> projectFindSkuInfoResps;
    /**
     * 活动详情
     */
    private String content;
    /**
     * 活动状态
     */
    private String status;
    /**
     * 策略类型
     */
    private String strategyType;
    /**
     * 策略ID
     */
    private Long strategyId;

    /** 构造ActivityTeamInfoResp  */
    public static ActivityTeamInfoResp teamInfoFindByIdBuild(ActivityTeamInfoSelectModel teamSelectModel) {

        return ActivityTeamInfoResp.builder()
                .teamId(teamSelectModel.getTeamId())
                .name(teamSelectModel.getName())
                .productId(teamSelectModel.getProductId())
                .productImage(teamSelectModel.getProductImage())
                .introduction(teamSelectModel.getIntroduction())
                .unit(teamSelectModel.getUnit())
                .imageList(teamSelectModel.getImageList())
                .endTime(teamSelectModel.getEndTime())
                .sort(teamSelectModel.getSort())
                .projectFindSkuInfoResps(
                        teamSelectModel.getActivityTeamProductSkuModels()
                                .stream().map(ProjectFindSkuInfoResp::projectFindSkuInfoBuild).toList())
                .content(teamSelectModel.getContent())
                .status(teamSelectModel.getStatus())
                .strategyType(teamSelectModel.getStrategyType())
                .strategyId(teamSelectModel.getStrategyId())
                .build();
    }

    /** 构造  */
    public static ActivityTeamInfoResp client(ActivityTeamInfo activityTeamInfo) {
        return ActivityTeamInfoResp.builder()
                .teamId(activityTeamInfo.getId())
                .name(activityTeamInfo.getName())
                .productId(activityTeamInfo.getProductId())
                .productImage(activityTeamInfo.getProductImage())
                .introduction(activityTeamInfo.getIntroduction())
                .unit(activityTeamInfo.getUnit())
                .imageList(activityTeamInfo.getImageList())
                .endTime(activityTeamInfo.getEndTime())
                .sort(activityTeamInfo.getSort())
                .content(activityTeamInfo.getContent())
                .status(activityTeamInfo.getStatus())
                .strategyType(activityTeamInfo.getStrategyType())
                .strategyId(activityTeamInfo.getStrategyId())
                .build();
    }
}
