package com.muyu.marketing.team.controller;


import com.github.pagehelper.PageInfo;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.PageUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.model.ActivityTeamInfoAddModel;
import com.muyu.marketing.domain.model.ActivityTeamInfoListModel;
import com.muyu.marketing.domain.model.ActivityTeamInfoUpdModel;
import com.muyu.marketing.domain.req.ActivityTeamInfoSaveReq;
import com.muyu.marketing.domain.req.ActivityTeamProductSkuInfoUpdReq;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.ActivityTeamInfoResp;
import com.muyu.marketing.domain.resp.TeamInfoListResp;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 营销团购活动控制层
 *
 * @author DongZeLiang
 * @date 2024-11-20 14:25
 */
@RestController
@RequestMapping("/team")
public class ActivityTeamController {

    @Autowired
    private ActivityTeamInfoService activityTeamInfoService;;


    /**
     * 客户端拼团列表
     */
    @PostMapping("/clientList")
    public Result<PageInfo<ActivityTeamInfoResp>> clientList(@RequestBody TeamInfoListReq teamInfoListReq){
        PageUtils.startPage(teamInfoListReq.getPageNum(),teamInfoListReq.getPageSize());
        List<ActivityTeamInfoResp> activityTeamInfoRespList=activityTeamInfoService.clientList(teamInfoListReq);
        PageInfo<ActivityTeamInfoResp> pageInfo = new PageInfo<>(activityTeamInfoRespList);
        return Result.success(pageInfo);

    }


    /**
     * 查询营销团购活动列表
     * @param teamInfoListReq 活动查询入参
     * @return 活动响应结果
     */
    @PostMapping("/list")
    public Result<TableDataInfo<TeamInfoListResp>> list(@RequestBody TeamInfoListReq teamInfoListReq) {
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = activityTeamInfoService.query(teamInfoListReq.buildQueryModel());
        List<TeamInfoListResp> respList = tableDataInfo.getRows().stream().map(TeamInfoListResp::listModelBuild).toList();
        return Result.success(
                new TableDataInfo<>(){{
                    setRows(respList);
                    setTotal(tableDataInfo.getTotal());
                }}
        );
    }


    /**
     * 添加团购活动
     * @param activityTeamInfoSaveReq 添加请求对象
     * @return 结果集
     */
    @PostMapping("/add")
    public Result<String> save(@RequestBody ActivityTeamInfoSaveReq activityTeamInfoSaveReq) {
        activityTeamInfoService.save(ActivityTeamInfoAddModel.addReqBuild(activityTeamInfoSaveReq));
        return Result.success();
    }


    /**
     * 回显拼团活动
     * @teamId 拼团活动id
     */
    @GetMapping("/getById/{teamId}")
    public Result<ActivityTeamInfoResp> getTeamInfo(@PathVariable("teamId") Long teamId){
        return Result.success(ActivityTeamInfoResp.teamInfoFindByIdBuild(activityTeamInfoService.findDetailById(teamId)));
    }


    /** 修改拼团活动*/
    @PutMapping
    private Result<String> update(ActivityTeamProductSkuInfoUpdReq activityTeamProductSkuInfoUpdReq){
        //转换拼团修改Model
        ActivityTeamInfoUpdModel activityTeamInfoUpdModel = ActivityTeamInfoUpdModel.reqConvertModel(activityTeamProductSkuInfoUpdReq);

        activityTeamInfoService.updateTeamInfo(activityTeamInfoUpdModel);

        return Result.success();

    }


}
