package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.model.*;
import com.muyu.marketing.domain.req.TeamInfoListReq;
import com.muyu.marketing.domain.resp.ActivityTeamInfoResp;
import com.muyu.marketing.team.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamOpenInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;

/**
 * 拼团活动
 */
@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo>
    implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query(ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {

//        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
//        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()), ActivityTeamInfo::getName, activityTeamInfoListQueryModel.getKeyWord());
//        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()), ActivityTeamInfo::getStatus, activityTeamInfoListQueryModel.getStatus());
//
//        /**
//         * Object<T> -> 创建对象的时候进行的占用
//         * <T> Result<T> 以方法返回值为占用
//         */
//        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
//        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
//        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
//                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
//                        (activityTeamInfoListModelBuilder) -> {
//                            TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
//                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
//                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
//                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());
//
//                            return activityTeamInfoListModelBuilder
//                                    .openTeamNumber(teamOpenTypeNumber)
//                                    .addTeamNumber(teamInTypeNumber)
//                                    .attendNumber(teamOpenTypeNumber + teamInTypeNumber)
//                                    .teamPrice(discountPrice.getTeamPrice())
//                                    .productPrice(discountPrice.getProductPrice())
//                                    .teamStock(teamProductStockModel.getTeamStock())
//                                    .remainStock(teamProductStockModel.getRemainStock())
//                                    .build();
//                        })).toList();
//        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
//        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
//        tableDataInfo.setRows(activityTeamInfoListModels);
//        return tableDataInfo;
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()), ActivityTeamInfo::getName, activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()), ActivityTeamInfo::getStatus, activityTeamInfoListQueryModel.getStatus());

        Page<ActivityTeamInfo> activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(), queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                        (activityTeamInfoListModelBuilder) -> {
                            TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                            TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                            Long teamOpenTypeNumber = activityTeamOpenInfoService.getTeamOpenTypeNumberByTeamId(activityTeamInfo.getId());
                            Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());

                            return activityTeamInfoListModelBuilder
                                    .openTeamNumber(teamOpenTypeNumber)
                                    .addTeamNumber(teamInTypeNumber)
                                    .attendNumber(teamOpenTypeNumber + teamInTypeNumber)
                                    .teamPrice(discountPrice.getTeamPrice())
                                    .productPrice(discountPrice.getProductPrice())
                                    .teamStock(teamProductStockModel.getTeamStock())
                                    .remainStock(teamProductStockModel.getRemainStock())
                                    .productId(activityTeamInfo.getProductId())
                                    .build();
                        })).toList();
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);
        return tableDataInfo;
    }

    /**
     * 通过添加模型，去进行商品拼团活动的操作
     *
     * @param activityTeamInfoAddModel 团购添加模型
     */
    @Override
    public void save(ActivityTeamInfoAddModel activityTeamInfoAddModel) {
        ActivityTeamInfo activityTeamInfo = ActivityTeamInfo.addModelBuild(activityTeamInfoAddModel);
        Assert.isTrue(this.save(activityTeamInfo), StringUtils.format("团购活动添加失败：[{}]", activityTeamInfoAddModel));
        List<ActivityTeamProductSkuAddModel> teamProductSkuAddModelList = activityTeamInfoAddModel.getActivityTeamProductSkuAddModelList();
        teamProductSkuAddModelList.forEach(activityTeamProductSkuAddModel -> activityTeamProductSkuAddModel.setTeamId(activityTeamInfo.getId()));
        this.activityTeamProductSkuInfoService.batchSave(teamProductSkuAddModelList);
    }

    /**
     * 查询拼团活动（回显）
     * @param teamId
     * @return
     */
    @Override
    public ActivityTeamInfoSelectModel findDetailById(Long teamId) {
        //根据拼团活动id 查询拼团活动
        ActivityTeamInfo activityTeamInfo = this.getById(teamId);
        //查询活动中的 商品sku集合
        List<ActivityTeamProductSkuModel> activityTeamProductSkuModelList = activityTeamProductSkuInfoService.findListByTeamId(teamId);
        return ActivityTeamInfoSelectModel.getTeamInfoSelectModel(activityTeamInfo,
                (teamInfoFindByIdRespModelBuilder) -> teamInfoFindByIdRespModelBuilder.activityTeamProductSkuModels(activityTeamProductSkuModelList).build()
        );
    }

    /**
     * 修改拼团活动活动
     */
    @Override
    public void updateTeamInfo(ActivityTeamInfoUpdModel activityTeamInfoUpdModel) {
        //查询当前活动状态
        ActivityTeamInfo activityTeamInfo = this.getById(activityTeamInfoUpdModel.getTeamId());
        if ("Y".equals(activityTeamInfo.getStatus())){
            //当前活动运营中禁止修改
            throw new RuntimeException("当前活动运营中禁止修改");
        }

        //修改:
        List<ActivityTeamProductSkuAddModel> skuAddList = activityTeamInfoUpdModel.getSkuList().getSkuAddList();
        //添加列表不为空并且长度大于零
        if (null != skuAddList && 0<skuAddList.size()){
            //构造 添加
            activityTeamProductSkuInfoService.batchSave(skuAddList);

        }

        List<ActivityTeamProductSkuUpdModel> skuUpdateList = activityTeamInfoUpdModel.getSkuList().getSkuUpdateList();
        //修改列表不为空并且长度大于零
        if (null != skuUpdateList && 0<skuUpdateList.size()){
            //构造 修改
            activityTeamProductSkuInfoService.updateBath(skuUpdateList);
        }

        List<Long> skuByIdDeleteList = activityTeamInfoUpdModel.getSkuList().getSkuByIdDeleteList();
        if (null != skuByIdDeleteList && 0<skuByIdDeleteList.size()){
            //构造 删除
            activityTeamProductSkuInfoService.deleteById(skuByIdDeleteList);
        }

    }

    /** 客户端列表*/
    @Override
    public List<ActivityTeamInfoResp> clientList(TeamInfoListReq teamInfoListReq) {
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        //查询活动状态
        if (null != teamInfoListReq.getStatus() && ! "".equals(teamInfoListReq.getStatus())){
            queryWrapper.eq(ActivityTeamInfo::getStatus,teamInfoListReq.getStatus());
        }
        if (null != teamInfoListReq.getKeyWord() && ! "".equals(teamInfoListReq.getKeyWord())){
            queryWrapper.eq(ActivityTeamInfo::getStatus,teamInfoListReq.getStatus());
        }
        //拼团列表
        List<ActivityTeamInfo> activityTeamInfos = this.list(queryWrapper);
        //构造成Resp
        ArrayList<ActivityTeamInfoResp> activityTeamInfoResps = new ArrayList<>();
        activityTeamInfos.forEach(activityTeamInfo -> {
            ActivityTeamInfoResp client = ActivityTeamInfoResp.client(activityTeamInfo);
            activityTeamInfoResps.add(client);
        });

        return activityTeamInfoResps;
    }


}
