package com.muyu.product.domain.resp;

import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.product.domain.model.ProjectAddModel;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import com.muyu.product.domain.req.ProjectInfoQueryReq;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.Date;
import java.util.function.Supplier;

/**
 * 商品信息对象 project_info
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class ProjectInfoListResp extends BaseEntity {
    /** 主键 */
    private Long id;

    /** 商品名称 */
    private String name;

    /** 商品描述 */
    private String introduction;


    /** 主类型 */
    private Long mianType;

    /**
     * 主类型名称
     */
    public String mianTypeName;
    /** 父类型 */
    private Long parentType;

    private String parentTypeName;
    /** 商品类型 */
    private Long type;

    private String typeName;
    /** 商品图片 */
    private String image;

    /** 商品轮播图 */
    private String carouselImages;

    /** 商品状态 */
    private String status;

    /** 规格 */
    private Long ruleId;
    private String ruleName;

    /** 品牌 */
    private Long brandId;
    private String brandName;





    /**
     * 查询构造器
     */
    public static ProjectInfoListResp queryBuild(ProjectInfoQueryReq projectInfoQueryReq){
        return ProjectInfoListResp.builder()
                .name(projectInfoQueryReq.getName())
                .introduction(projectInfoQueryReq.getIntroduction())
                .mianType(projectInfoQueryReq.getMianType())
                .parentType(projectInfoQueryReq.getParentType())
                .type(projectInfoQueryReq.getType())
                .image(projectInfoQueryReq.getImage())
                .carouselImages(projectInfoQueryReq.getCarouselImages())
                .status(projectInfoQueryReq.getStatus())
                .ruleId(projectInfoQueryReq.getRuleId())
                .brandId(projectInfoQueryReq.getBrandId())
                .build();
    }

    /**
     * 添加构造器
     */
    public static ProjectInfoListResp saveModelBuild(ProjectAddModel projectAddModel, Supplier<String> createBy){
        return ProjectInfoListResp.builder()
                .name(projectAddModel.getName())
                .introduction(projectAddModel.getIntroduction())
                .mianType(projectAddModel.getMianType())
                .parentType(projectAddModel.getParentType())
                .type(projectAddModel.getType())
                .image(projectAddModel.getImage())
                .carouselImages(projectAddModel.getCarouselImages())
                .status(projectAddModel.getStatus())
                .ruleId(projectAddModel.getRuleId())
                .brandId(projectAddModel.getBrandId())
                .createBy(createBy.get())
                .createTime(new Date())
                .build();
    }

    /**
     * 修改构造器
     */
    public static ProjectInfoListResp editBuild(Long id, ProjectInfoEditReq projectInfoEditReq){
        return ProjectInfoListResp.builder()
                .id(id)
                .name(projectInfoEditReq.getName())
                .introduction(projectInfoEditReq.getIntroduction())
                .mianType(projectInfoEditReq.getMianType())
                .parentType(projectInfoEditReq.getParentType())
                .type(projectInfoEditReq.getType())
                .image(projectInfoEditReq.getImage())
                .carouselImages(projectInfoEditReq.getCarouselImages())
                .status(projectInfoEditReq.getStatus())
                .ruleId(projectInfoEditReq.getRuleId())
                .brandId(projectInfoEditReq.getBrandId())
                .build();
    }

}
