package com.muyu.product.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageInfo;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.product.domain.AsAttributeGroup;
import com.muyu.product.domain.AttributeGroup;
import com.muyu.product.domain.AttributeInfo;
import com.muyu.product.domain.model.AttributeGroupSaveModel;
import com.muyu.product.domain.req.AttributeGroupEditReq;
import com.muyu.product.domain.req.AttributeGroupStateReq;
import com.muyu.product.domain.resp.AttributeGroupPageResp;
import com.muyu.product.domain.resp.AttributeGroupUpd;
import com.muyu.product.domain.resp.AttributeGroupUpdResp;
import com.muyu.product.mapper.AsAttributeGroupMapper;
import com.muyu.product.mapper.AttributeGroupMapper;
import com.muyu.product.service.AsAttributeGroupService;
import com.muyu.product.service.AttributeGroupService;
import com.muyu.product.service.AttributeInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 属性组Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class AttributeGroupServiceImpl extends ServiceImpl<AttributeGroupMapper, AttributeGroup> implements AttributeGroupService {

    @Autowired
    private AsAttributeGroupService asAttributeGroupService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    @Autowired
    private AsAttributeGroupMapper attributeGroupMapper;

    /**
     * 分页查询
     *
     * @param attributeGroupQuery 分组对象
     *
     * @return 返回结果
     */
    @Override
    public TableDataInfo<AttributeGroupPageResp> page (AttributeGroup attributeGroupQuery) {
        List<AttributeGroup> list = this.list(attributeGroupQuery);
        List<AttributeGroupPageResp> pageRespList = list.stream()
                .map(attributeGroup ->
                        AttributeGroupPageResp.groupFunBuild(
                                attributeGroup, groupId -> attributeInfoService.attributeListByGroupId(groupId)
                        )
                )
                .toList();
        return TableDataInfo.<AttributeGroupPageResp>builder()
                .total(new PageInfo(list).getTotal())
                .rows(pageRespList)
                .build();
    }

    /**
     * 查询属性组列表
     *
     * @param attributeGroup 属性组
     *
     * @return 属性组
     */
    @Override
    public List<AttributeGroup> list (AttributeGroup attributeGroup) {
        LambdaQueryWrapper<AttributeGroup> queryWrapper = new LambdaQueryWrapper<>();

        if (ObjUtils.notNull(attributeGroup.getName())) {
            queryWrapper.like(AttributeGroup::getName, attributeGroup.getName());
        }

        if (ObjUtils.notNull(attributeGroup.getStates())) {
            queryWrapper.eq(AttributeGroup::getStates, attributeGroup.getStates());
        }

        return list(queryWrapper);
    }

    /**
     * 保存
     *
     * @param attributeGroupSaveModel 属性组保存模型
     *
     * @return 是否成功
     */
    @Override
    @Transactional
    public Boolean save (AttributeGroupSaveModel attributeGroupSaveModel) {
        AttributeGroup attributeGroup = attributeGroupSaveModel.buildAttributeGroup();
        boolean save = this.save(attributeGroup);
        // 处理属性ID
        Long attributeGroupId = attributeGroup.getId();
        List<Long> attributeIdList = attributeGroupSaveModel.getAttributeIdList();

        asAttributeGroupService.saveBatch(
                attributeIdList.stream()
                        .map(attributeId -> AsAttributeGroup.buildGroup(attributeGroupId, attributeId))
                        .toList()
        );
        return save;
    }

    @Override
    public AttributeGroupUpdResp getUpdById(Long id) {
        AttributeGroup attributeGroup = this.getById(id);

        LambdaQueryWrapper<AsAttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AsAttributeGroup::getGroupId, id);
        List<AsAttributeGroup> asAttributeGroupList = asAttributeGroupService.list(queryWrapper);
        List<Long> attributeIdList = asAttributeGroupList.stream()
                .map(AsAttributeGroup::getAttributeId)
                .toList();
        List<AttributeInfo> attributeInfoList = attributeInfoService.listByIds(attributeIdList);

        return AttributeGroupUpdResp.builder()
                .id(attributeGroup.getId())
                .name(attributeGroup.getName())
                .states(attributeGroup.getStates())
                .attributeList(attributeInfoList)
                .build();
    }



    /**
     * 吴彦祖 的 修改 属性
     */
    @Override
    public void updateAttribute(Long id, AttributeGroupEditReq attributeGroupEditReq) {
        //删除原 属性组
        attributeGroupMapper.delete(id);
        if (null == attributeGroupEditReq.getAttributeList() || 0==attributeGroupEditReq.getAttributeList().size()){
            List<Long> list = attributeGroupEditReq.getAttributeIdList();
            //添加
            for (Long info : list) {
                attributeGroupMapper.add(id,info);
            }
        }else {
            List<AttributeInfo> list = attributeGroupEditReq.getAttributeList();
            //添加
            for (AttributeInfo info : list) {
                attributeGroupMapper.add(id,info.getId());
            }
        }
        //获取新数据组

    }

    @Autowired
    private AttributeGroupMapper mapper;

    /**
     *  吴彦祖 修改 属性 状态
     * @param attributeGroupStateReq
     */
    @Override
    public void updateState(AttributeGroupStateReq attributeGroupStateReq) {
        LambdaQueryWrapper<AttributeGroup> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AttributeGroup ::getId ,attributeGroupStateReq.getId());

        AttributeGroup attributeGroup = new AttributeGroup();
        attributeGroup.setStates(attributeGroupStateReq.getStates());

        mapper.update(attributeGroup,queryWrapper);
    }

    //属性与组中间列表
    @Autowired
    private AsAttributeGroupService attributeGroupService;

    /**
     * 吴彦祖 的 回显属性组
     * @param id
     * @return
     */
    @Override
    public AttributeGroupUpd getByIdUpd(Long id) {
        //获取当前id的  组
        AttributeGroup byId = this.getById(id);
        //查询 该 组 的 属性 集合
        LambdaQueryWrapper<AsAttributeGroup> wrapper = new LambdaQueryWrapper<>();
        //返回
        wrapper.eq(AsAttributeGroup::getGroupId,id);
        List<AsAttributeGroup> asAttributeGroups=attributeGroupService.list(wrapper);
        //判断当前会显得 属性及 是否为空
        if (asAttributeGroups.size() != 0){
            List<Long> longList = asAttributeGroups.stream()
                    .map(AsAttributeGroup::getAttributeId)//属性
                    .toList();
            List<AttributeInfo> attributeInfos = attributeInfoService.listByIds(longList);
            return AttributeGroupUpd.builder()
                    .id(byId.getId())
                    .name(byId.getName())
                    .attributeList(//属性
                            attributeInfos
                    ).build();//修改回显时信息
        }

        return AttributeGroupUpd.builder()
                .id(byId.getId())
                .name(byId.getName())
                .build();//修改回显时信息
    }

}
